/* --------------------------------------------------------------------------
** 
** 	Funzioni per la gestione dell'UART del PIC 16F628 ed equivalenti
** 
**	File  : UART628.H	Ver.: 1.0	Ultima modifica: 07/03/05
**		
**	Autore: Roberto D'Amico (iw0gtf@libero.it)
**
**	Porte usate per default:
**	PORTB bit 1 RX (IN)
**	PORTA bit 2 TX (OUT)
**	
**	Per usare queste routine:
**	- settare le porte RB1 come input e RB2 come output
**	- modificare le impostazioni di uart_init() in base al quarzo utilizzato
**
**      NOTE
**	Per ulteriori valori di SPBRG consultare la tabella sul DataSheet
**	
**	
** ----------------------------------------------------------------------- */

/* Impostazione del Baud rate generator in funzione dei Baud desiderati e la
   frequenza del quarzo BRGH=1. I valori sono indicati sul datasheet */
#define BRG_9600_4		0x19
#define BRG_9600_20		0x81

/****************************************************************************
*** Inizializza la USART in modalit asincrona						  	  ***
****************************************************************************/
void uart_init()
{
unsigned char i;
unsigned char buffer;

	SPBRG=BRG_9600_4;	// Baud Rate Generator - BRGH=1,Fosc=4,SYNC=0
	TXSTA=0x24;
	RCSTA=0x90;

	buffer=RCREG;		// Svuota il buffer di ricezione
	buffer=RCREG;
	buffer=RCREG;
}
/****************************************************************************
*** Trasmette un carattere tramite la UART							  	  ***
****************************************************************************/
void uart_putch(unsigned char char_to_send)
{
	while(!TRMT);			// Polling per Tx buffer empty
	TXREG=char_to_send;
}
/****************************************************************************
*** Trasmette una stringa tramite la UART							  	  ***
****************************************************************************/
void uart_puts(const char *s)
{
	while(*s)
		uart_putch(*s++);	// loop di trasmissione

	//Ser_send(0x0A);		// attivare se serve CR e/o LF
	//Ser_send(0x0D);
}
/****************************************************************************
*** Riceve un carattere tramite la UART							  	 	  ***
****************************************************************************/
unsigned char uart_getch()
{
	while(!RCIF);			// Polling per Rx buffer full
	return RCREG;
}
